package minetweaker.mc1112.item;

import minetweaker.MineTweakerAPI;
import minetweaker.api.block.IBlock;
import minetweaker.api.data.*;
import minetweaker.api.item.*;
import minetweaker.api.liquid.ILiquidStack;
import minetweaker.api.oredict.IOreDictEntry;
import minetweaker.api.player.IPlayer;
import minetweaker.mc1112.actions.*;
import minetweaker.mc1112.block.MCItemBlock;
import minetweaker.mc1112.data.NBTConverter;
import minetweaker.mc1112.liquid.MCLiquidStack;
import minetweaker.util.ArrayUtil;
import net.minecraft.block.Block;
import net.minecraft.item.*;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fluids.*;
import net.minecraftforge.fml.relauncher.ReflectionHelper;
import net.minecraftforge.oredict.OreDictionary;

import java.util.*;

import static minetweaker.api.minecraft.MineTweakerMC.getItemStack;

/**
 * @author Stan
 */
public class MCItemStack implements IItemStack {
    
    private final ItemStack stack;
    private final List<IItemStack> items;
    private IData tag = null;
    private boolean wildcardSize;
    
    public MCItemStack(ItemStack itemStack) {
        if(itemStack.func_190926_b())
            throw new IllegalArgumentException("stack cannot be null");
        
        stack = itemStack.func_77946_l();
        items = Collections.singletonList(this);
    }
    
    public MCItemStack(ItemStack itemStack, boolean wildcardSize) {
        this(itemStack);
        
        this.wildcardSize = wildcardSize;
    }
    
    private MCItemStack(ItemStack itemStack, IData tag) {
        if(itemStack.func_190926_b())
            throw new IllegalArgumentException("stack cannot be null");
        
        stack = itemStack;
        items = Collections.singletonList(this);
        this.tag = tag;
    }
    
    private MCItemStack(ItemStack itemStack, IData tag, boolean wildcardSize) {
        if(itemStack.func_190926_b())
            throw new IllegalArgumentException("stack cannot be null");
        stack = itemStack;
        items = Collections.singletonList(this);
        this.tag = tag;
        this.wildcardSize = wildcardSize;
    }
    
    @Override
    public IItemDefinition getDefinition() {
        return new MCItemDefinition(Item.field_150901_e.func_177774_c(stack.func_77973_b()).toString(), stack.func_77973_b());
    }
    
    @Override
    public String getName() {
        return stack.func_77977_a();
    }
    
    @Override
    public String getDisplayName() {
        return stack.func_82833_r();
    }
    
    @Override
    public void setDisplayName(String name) {
        MineTweakerAPI.apply(new SetTranslationAction(getName() + ".name", name));
    }
    
    @Override
    public int getMaxStackSize() {
        return stack.func_77976_d();
    }
    
    @Override
    public void setMaxStackSize(int size) {
        MineTweakerAPI.apply(new SetStackSizeAction((ItemStack) getInternal(), size));
    }
    
    @Override
    public float getBlockHardness() {
        return ReflectionHelper.getPrivateValue(Block.class, Block.func_149634_a(stack.func_77973_b()), "blockHardness");
    }
    
    @Override
    public void setBlockHardness(float hardness) {
        MineTweakerAPI.apply(new SetBlockHardnessAction(stack, hardness));
    }
    
    @Override
    public int getDamage() {
        return stack.func_77952_i();
    }
    
    @Override
    public IData getTag() {
        if(tag == null) {
            if(stack.func_77978_p() == null) {
                return DataMap.EMPTY;
            }
            
            tag = NBTConverter.from(stack.func_77978_p(), true);
        }
        return tag;
    }
    
    @Override
    public int getMaxDamage() {
        return stack.func_77958_k();
    }
    
    @Override
    public void setMaxDamage(int damage) {
        MineTweakerAPI.apply(new SetStackmaxDamageAction(stack, damage));
    }
    
    @Override
    public ILiquidStack getLiquid() {
        FluidStack liquid = FluidUtil.getFluidContained(stack);
        return liquid == null ? null : new MCLiquidStack(liquid);
    }
    
    @Override
    public IIngredient anyDamage() {
        if(stack.func_77973_b().func_77614_k()) {
            MineTweakerAPI.logWarning("subitems don't have damaged states");
            return this;
        } else {
            ItemStack result = new ItemStack(stack.func_77973_b(), stack.func_190916_E(), OreDictionary.WILDCARD_VALUE);
            result.func_77982_d(stack.func_77978_p());
            return new MCItemStack(result, tag);
        }
    }
    
    @Override
    public IItemStack withDamage(int damage) {
        if(stack.func_77973_b().func_77614_k()) {
            MineTweakerAPI.logWarning("subitems don't have damaged states");
            return this;
        } else {
            ItemStack result = new ItemStack(stack.func_77973_b(), stack.func_190916_E(), damage);
            result.func_77982_d(stack.func_77978_p());
            return new MCItemStack(result, tag);
        }
    }
    
    @Override
    public IItemStack anyAmount() {
        ItemStack result = new ItemStack(stack.func_77973_b(), 1, stack.func_77952_i());
        result.func_77982_d(stack.func_77978_p());
        return new MCItemStack(result, tag, true);
    }
    
    @Override
    public IItemStack withAmount(int amount) {
        ItemStack result = new ItemStack(stack.func_77973_b(), amount, stack.func_77952_i());
        result.func_77982_d(stack.func_77978_p());
        return new MCItemStack(result, tag);
    }
    
    @Override
    public IItemStack withTag(IData tag) {
        ItemStack result = new ItemStack(stack.func_77973_b(), stack.func_190916_E(), stack.func_77952_i());
        if(tag == null) {
            result.func_77982_d(null);
        } else {
            result.func_77982_d((NBTTagCompound) NBTConverter.from(tag));
        }
        return new MCItemStack(result, tag);
    }
    
    @Override
    public IItemStack withEmptyTag() {
        ItemStack result = new ItemStack(stack.func_77973_b(), stack.func_190916_E(), stack.func_77952_i());
        result.func_77982_d(new NBTTagCompound());
        return new MCItemStack(result, NBTConverter.from(new NBTTagCompound(), true));
    }
    
    @Override
    public IItemStack updateTag(IData tagUpdate) {
        if(tag == null) {
            if(stack.func_77978_p() == null) {
                return withTag(tagUpdate);
            }
            
            tag = NBTConverter.from(stack.func_77978_p(), true);
        }
        
        IData updated = tag.update(tagUpdate);
        return withTag(updated);
    }
    
    @Override
    public IItemStack removeTag(String tag) {
        ItemStack result = new ItemStack(stack.func_77973_b(), stack.func_190916_E(), stack.func_77952_i());
        
        if(tag == null) {
            result.func_77982_d(null);
        } else {
            result.func_77978_p().func_82580_o(tag);
        }
        IData dataTag = NBTConverter.from(result.func_77978_p(), false);
        return new MCItemStack(result, dataTag);
    }
    
    @Override
    public String getMark() {
        return null;
    }
    
    @Override
    public int getAmount() {
        return stack.func_190916_E();
    }
    
    @Override
    public List<IItemStack> getItems() {
        return items;
    }
    
    @Override
    public List<ILiquidStack> getLiquids() {
        return Collections.emptyList();
    }
    
    @Override
    public IItemStack amount(int amount) {
        return withAmount(amount);
    }
    
    @Override
    public WeightedItemStack percent(float chance) {
        return new WeightedItemStack(this, chance * 0.01f);
    }
    
    @Override
    public WeightedItemStack weight(float chance) {
        return new WeightedItemStack(this, chance);
    }
    
    @Override
    public IIngredient transform(IItemTransformer transformer) {
        return new IngredientItem(this, null, ArrayUtil.EMPTY_CONDITIONS, new IItemTransformer[]{transformer});
    }
    
    @Override
    public IIngredient only(IItemCondition condition) {
        return new IngredientItem(this, null, new IItemCondition[]{condition}, ArrayUtil.EMPTY_TRANSFORMERS);
    }
    
    @Override
    public IIngredient marked(String mark) {
        return new IngredientItem(this, mark, ArrayUtil.EMPTY_CONDITIONS, ArrayUtil.EMPTY_TRANSFORMERS);
    }
    
    @Override
    public IIngredient or(IIngredient ingredient) {
        return new IngredientOr(this, ingredient);
    }
    
    @Override
    public boolean matches(IItemStack item) {
        ItemStack internal = getItemStack(item);
        if(stack.func_77942_o()) {
            return matchesExact(item);
        }
        return internal != null && stack != null && internal.func_77973_b() == stack.func_77973_b() && (wildcardSize || internal.func_190916_E() >= stack.func_190916_E()) && (stack.func_77952_i() == OreDictionary.WILDCARD_VALUE || stack.func_77952_i() == internal.func_77952_i() || (!stack.func_77981_g() && !stack.func_77973_b().func_77645_m()));
    }
    
    @Override
    public boolean matchesExact(IItemStack item) {
        ItemStack internal = getItemStack(item);
        if(internal.func_77978_p() != null && stack.func_77978_p() == null) {
            return false;
        }
        if(internal.func_77978_p() == null && stack.func_77978_p() != null) {
            return false;
        }
        if(internal.func_77978_p() == null && stack.func_77978_p() == null) {
            return stack.func_77973_b() == internal.func_77973_b() && (internal.func_77960_j() == 32767 || stack.func_77960_j() == internal.func_77960_j());
        }
        if(internal.func_77978_p().func_150296_c().equals(stack.func_77978_p().func_150296_c())) {
            for(String s : internal.func_77978_p().func_150296_c()) {
                if(!internal.func_77978_p().func_74781_a(s).equals(stack.func_77978_p().func_74781_a(s))) {
                    return false;
                }
            }
        }
        return stack.func_77973_b() == internal.func_77973_b() && (internal.func_77960_j() == 32767 || stack.func_77960_j() == internal.func_77960_j());
    }
    
    @Override
    public boolean matches(ILiquidStack liquid) {
        return false;
    }
    
    @Override
    public boolean contains(IIngredient ingredient) {
        List<IItemStack> iitems = ingredient.getItems();
        return !(iitems == null || iitems.size() != 1) && matches(iitems.get(0));
    }
    
    @Override
    public IItemStack applyTransform(IItemStack item, IPlayer byPlayer) {
        return item;
    }
    
    @Override
    public boolean hasTransformers() {
        return false;
    }
    
    @Override
    public Object getInternal() {
        return stack;
    }
    
    @Override
    public IBlock asBlock() {
        ResourceLocation name = Block.field_149771_c.func_177774_c(Block.func_149634_a(stack.func_77973_b()));
        if(Block.field_149771_c.func_148741_d(name)) {
            return new MCItemBlock(stack);
        } else {
            throw new ClassCastException("This item is not a block");
        }
    }
    
    @Override
    public List<IOreDictEntry> getOres() {
        List<IOreDictEntry> result = new ArrayList<>();
        
        for(String key : OreDictionary.getOreNames()) {
            for(ItemStack is : OreDictionary.getOres(key)) {
                if(is.func_77973_b() == stack.func_77973_b() && (is.func_77952_i() == OreDictionary.WILDCARD_VALUE || is.func_77952_i() == stack.func_77952_i())) {
                    result.add(MineTweakerAPI.oreDict.get(key));
                    break;
                }
            }
        }
        
        return result;
    }
    
    // #############################
    // ### Object implementation ###
    // #############################
    
    @Override
    public int hashCode() {
        int hash = 7;
        hash = 41 * hash + stack.func_77973_b().hashCode();
        hash = 41 * hash + stack.func_77952_i();
        hash = 41 * hash + stack.func_190916_E();
        hash = 41 * hash + (stack.func_77978_p() == null ? 0 : stack.func_77978_p().hashCode());
        hash = 41 * hash + (this.wildcardSize ? 1 : 0);
        return hash;
    }
    
    @Override
    public boolean equals(Object obj) {
        if(obj == null) {
            return false;
        }
        if(getClass() != obj.getClass()) {
            return false;
        }
        final MCItemStack other = (MCItemStack) obj;
        return this.stack.func_77973_b() == other.stack.func_77973_b() && this.stack.func_77952_i() == other.stack.func_77952_i() && this.stack.func_190916_E() == other.stack.func_190916_E() && !(this.stack.func_77978_p() != other.stack.func_77978_p() && (this.stack == null || this.stack.equals(other.stack))) && this.wildcardSize == other.wildcardSize;
    }
    
    @Override
    public String toString() {
        StringBuilder result = new StringBuilder();
        result.append('<');
        result.append(Item.field_150901_e.func_177774_c(stack.func_77973_b()));
        
        if(stack.func_77952_i() == OreDictionary.WILDCARD_VALUE) {
            result.append(":*");
        } else if(stack.func_77952_i() > 0) {
            result.append(':').append(stack.func_77952_i());
        }
        result.append('>');
        
        if(stack.func_77978_p() != null) {
            result.append(".withTag(");
            result.append(NBTConverter.from(stack.func_77978_p(), wildcardSize).toString());
            result.append(")");
        }
        
        return result.toString();
    }
    
    
}
